#include <stdio.h>
#include <math.h>
#include <avs/flow.h> 
#include <avs/avs_data.h>
#include <avs/field.h>
#include <time.h>

extern float bscall();
#define PERFRAME 30

AVSinit_modules()
{
  int fin();

  AVSmodule_from_desc(fin);
}

/*  The routine "fin" is the description routine.  */
fin()
{
  int fin_compute();	 /* declare the compute function (below) */
  int in_port,out_port;	 /* temporary to hold the port number */
  int parm;		 /* temporary to hold the parm number */

  /* Set the module name and type */
  AVSset_module_name("BS_DEC", MODULE_MAPPER);
  
  /* Create an input port for the required field input */
  in_port = AVScreate_input_port("Input Field",
				 "field 1D uniform scalar float", REQUIRED);  
  /* Create output ports for the result */
  out_port = AVScreate_output_port("Field Output", 
				   "field 1D scalar uniform float");
  out_port = AVScreate_output_port("BS_bias", "real");  

  /* Add parameter:  */
  parm = AVSadd_float_parameter("V_BS", 1.0, 0.0, 5.0);
  parm = AVSadd_parameter("points/frame","integer", PERFRAME, 1, 100);
  
  /* Tell avs what subroutine to call to do the compute */
  AVSset_compute_proc(fin_compute);
}

/*
 * fin_compute is the compute routine.  It is called whenever AVS wants to 
 * compute new model results.  The arguments are: the value of the input
 * field, the new output field (doubly indirected), the parameter 
 * values in the order same as declaration above.
 * Note the order is always inputs, outputs, parameters. 
 */

fin_compute(input, output,pbias, pv_bs,size)
AVSfield_float *input, **output;
float *pv_bs,**pbias;
int size;
{
  register int i;
  float v_bs = *pv_bs;
  
  AVSfield_float *temp_field = NULL;
  static float bias;
  int dims[1];
   
  float sz,szdiv,k,rate,t,tdiv,div,call;
  double exp();
  float *dataptr = input->data;

  sz = *(dataptr++);  k = *(dataptr++); rate = *(dataptr++);
  t = *(dataptr++); tdiv = *(dataptr++); div = *(dataptr++);
  call = *dataptr;
  
  if (sz<0.0) sz = -sz;
  /* Allocate space for output */
  dims[0] = size;
  temp_field = (AVSfield_float *)AVSdata_alloc
    ("field 1D scalar uniform float",dims);
  if (temp_field == NULL) {
    AVSerror("Allocation of output field failed.");
    return(0);
  }
     
  if (*output) {
    for (i=0; i<(size-1); i++) 
      temp_field->data[i] = (*output)->data[i+1];
    AVSdata_free("field",*output);
  }

  szdiv = sz - div/exp(rate*tdiv);
  temp_field->data[size-1] = bias = 
    bscall(szdiv,k,rate, t, v_bs) - call ;

  fprintf(stderr," B-S bias=%8.5f",bias);
  *output = temp_field;
  *pbias = &bias; 
  
  /* When we're done, we return 1 to indicate success */
  return(1);
}
